/////////////////////////////////////////////////////////////////////////
//
// © University of Southampton IT Innovation Centre, 2017
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road, 
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Simon Crowle
//      Created Date :          19/12/2017
//      Created for Project :   FLAME
//
/////////////////////////////////////////////////////////////////////////
package uk.ac.soton.innovation.flame.clmc.monspec.test.base;

import java.util.ArrayList;
import java.util.List;
import org.influxdb.InfluxDB;
import org.junit.After;
import org.junit.AfterClass;
import static org.junit.Assert.assertTrue;
import org.junit.Before;
import org.junit.BeforeClass;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * BaseTest is an abstract class used to set up InfluxDB resource before
 * executing tests proper. Derive from this class and use protected methods.
 */
public abstract class BaseTest 
{
    protected Logger   logger = LoggerFactory.getLogger( BaseTest.class );
    protected InfluxDB influxDB;
    
    public BaseTest() 
    {}
    
    @BeforeClass
    public static void setUpClass() 
    {}
    
    @AfterClass
    public static void tearDownClass() 
    {}
    
    @Before
    public void setUp()
    {
        try
        {
            // Create test database if it does not already exist
            influxDB = InfluxUtility.getInfluxDBConnection();
            String dbName = InfluxUtility.getTestConfig().getProperty( "influxDB_DB" );
            
            if ( !influxDB.databaseExists(dbName) )
                influxDB.createDatabase( dbName );
            
            influxDB.setDatabase( dbName );
        }
        catch ( Exception ex )
        {
            logger.error( "Failed to get InfluxDB connection: " + ex.getMessage() );
            assertTrue( false );
        }
    }
    
    @After
    public void tearDown() 
    {}
    
    // Protected methods -------------------------------------------------------
    /**
     * Attempts to write InfluxDB Line Protocol lines to InfluxDB
     * 
     * @param seriesLabel - label of input resource (see /src/main/resources/inputs
     */
    protected void writeProtocolLines( String seriesLabel )
    {
        try
        {
            // Get line protocols (will throw if non-existent or empty)
            List<String> protocols = InfluxUtility.getInputSeries( seriesLabel );
            
            for ( String lp : protocols )
                influxDB.write( lp );
        }
        catch ( Exception ex )
        {
            String err = "Failed writing protocol lines: " + ex.getMessage();
            logger.error( err );
            
            assertTrue( false );
        }
    }
    
    /**
     * Attempts to execute one or more queries with InfluxDB
     * 
     * @param queryLabel - name of query input file resource (see /src/main/resources/queries)
     * @param printOut   - boolean to indicate whether results are logged out to console
     * @return           - Array of query results (per series set)
     */
    protected List<List<String>> executeQueries( String queryLabel, boolean printOut )
    {
        List<List<String>> uberResult = new ArrayList<>();
        
        try
        {
            // Get test query set (will throw if non-existent or empty)
            List<String> queries = InfluxUtility.getQueries( queryLabel );
            
            for ( String q : queries )
                uberResult.add(InfluxUtility.executeQuery(q,printOut) );
        }
        catch ( Exception ex )
        {
            String err = "Failed to execute queries: " + queryLabel;
            logger.error( err );
            
            assertTrue( false );
        }
        
        return uberResult;
    }
}
